<?php

namespace Drupal\media_alias_display\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\Controller\EntityViewController;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Path\CurrentPathStack;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StreamWrapper\StreamWrapperManagerInterface;
use Drupal\file\Entity\File;
use Drupal\path_alias\AliasManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\Core\Url;
use Drupal\media\Entity\Media;

/**
 * Defines a controller to render a file with Media Alias being used.
 */
class DisplayController extends EntityViewController {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected AccountInterface $currentUser;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected LoggerChannelFactoryInterface $loggerFactory;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected Request $request;

  /**
   * The current path.
   *
   * @var \Drupal\Core\Path\CurrentPathStack
   */
  protected CurrentPathStack $currentPath;

  /**
   * The path alias manager.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected AliasManagerInterface $aliasManager;

  /**
   * Drupal\Core\Config\ConfigManagerInterface definition.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configManager;

  /**
   * The stream wrapper manager.
   *
   * @var \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface
   */
  protected StreamWrapperManagerInterface $streamWrapperManager;

  /**
   * The controller constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   Current user.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $loggerFactory
   *   The logger factory.
   * @param \Symfony\Component\HttpFoundation\Request $request_stack
   *   Request stack.
   * @param \Drupal\Core\Path\CurrentPathStack $current_path
   *   The current path.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   The path alias manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config
   *   Configuration Interface.
   * @param \Drupal\Core\StreamWrapper\StreamWrapperManagerInterface $stream_wrapper_manager
   *   The stream wrapper manager.
   */
  public function __construct(EntityTypeManagerInterface    $entity_type_manager,
                              RendererInterface             $renderer,
                              AccountInterface              $current_user,
                              LoggerChannelFactoryInterface $loggerFactory,
                              Request                       $request_stack,
                              CurrentPathStack              $current_path,
                              AliasManagerInterface         $alias_manager,
                              ConfigFactoryInterface        $config,
                              StreamWrapperManagerInterface $stream_wrapper_manager
  ) {
    parent::__construct($entity_type_manager, $renderer);
    $this->currentUser = $current_user;
    $this->loggerFactory = $loggerFactory;
    $this->request = $request_stack;
    $this->currentPath = $current_path;
    $this->aliasManager = $alias_manager;
    $this->configManager = $config;
    $this->streamWrapperManager = $stream_wrapper_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('renderer'),
      $container->get('current_user'),
      $container->get('logger.factory'),
      $container->get('request_stack')->getCurrentRequest(),
      $container->get('path.current'),
      $container->get('path_alias.manager'),
      $container->get('config.factory'),
      $container->get('stream_wrapper_manager'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function view(EntityInterface $media, $view_mode = 'full', $langcode = NULL) {
    $config = $this->configManager->get('media_alias_display.settings');

    if (!empty($config->get('kill_switch')) && $config->get('kill_switch')) {
      return parent::view($media, $view_mode);
    }

    $allowed_bundles = $config->get('media_bundles');
    if (!empty($config->get('media_bundles'))) {
      $allowAllBundles = TRUE;
      foreach ($config->get('media_bundles') as $bundle) {
        if ($bundle !== 0) {
          $allowAllBundles = FALSE;
          break;
        }
      }

      if (isset($allowed_bundles[$media->bundle()]) && $allowed_bundles[$media->bundle()] === 0 && !$allowAllBundles) {
        return parent::view($media, $view_mode);
      }
    }

    $current_path = $this->currentPath->getPath();
    $current_alias = $this->request->getRequestUri();

    $alias = $this->aliasManager->getPathByAlias($current_path);
    $params = Url::fromUri("internal:" . $alias)->getRouteParameters();
    $entity_type = key($params);
    $mid = $params[$entity_type];
    $media = Media::load($mid);

    if ($media == NULL) {
      $this->loggerFactory->get('media_alias_display')
        ->notice("Can't find media object for @path", [
          '@path' => $current_path,
        ]);
      throw new NotFoundHttpException("Can't find media object.");
    }

    $bundle = $media->bundle();
    $edit_own = 'edit own ' . $bundle . ' media';
    $edit_any = 'edit any ' . $bundle . ' media';

    // Skip redirect and go straight to media object.
    if (str_contains($current_alias, "edit-media") && ($this->currentUser->hasPermission($edit_own) || $this->currentUser->hasPermission($edit_any))) {
      return new RedirectResponse('/media/' . $mid . '/edit');
    }

    if (\Drupal::moduleHandler()->moduleExists('media_alias_display_field_override')) {
      $override_module = $media->get("field_override_mad_module")->value;
      if (isset($override_module) && $override_module) {
        return parent::view($media, $view_mode);
      }
    }

    $source = $media->getSource();
    $config = $source->getConfiguration();
    $field = $config['source_field'];

    $fid = $media->{$field}->target_id;

    // If media has no file item.
    if (!$fid) {
      $this->loggerFactory->get('media_alias_display')
        ->notice("The media item requested has no file referenced/uploaded for @path", [
          '@path' => $current_path,
        ]);
      return parent::view($media, $view_mode);
    }

    $file = File::load($fid);

    // Or file entity could not be loaded.
    if (!$file) {
      $this->loggerFactory->get('media_alias_display')
        ->notice("File id could not be loaded for " . $current_path);
      return parent::view($media, $view_mode);
    }

    $uri = $file->getFileUri();
    $scheme = $this->streamWrapperManager->getScheme($uri);

    // Or item does not exist on disk.
    if (!$this->streamWrapperManager->isValidScheme($scheme) || !file_exists($uri)) {
      $this->loggerFactory->get('media_alias_display')
        ->notice("File does not exist for @path", [
          '@path' => $current_path,
        ]);
      return parent::view($media, $view_mode);
    }

    $filename = $file->getFilename();

    $response = new BinaryFileResponse($uri);
    $response->setContentDisposition(
      ResponseHeaderBag::DISPOSITION_INLINE,
      $filename
    );

    return new BinaryFileResponse($uri, Response::HTTP_OK, [], $scheme !== 'private');
  }

}
